from sqlalchemy import Column, Integer, Table
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy.dialects.mysql import TIMESTAMP, VARCHAR, LONGTEXT
# from sqlalchemy.ext.declarative import declarative_base

Base_Model = declarative_base()


def create_model_performance(table_name):
    base_table = Table(
        table_name,
        Base_Model.metadata,
        Column("pk_id", Integer, primary_key=True,
               autoincrement=True, nullable=False),
        Column("customer_id", VARCHAR(20)),
        Column("training_id", VARCHAR(45)),
        Column("model_id", VARCHAR(20)),
        Column("profile_id", VARCHAR(20)),
        Column("tag", VARCHAR(20)),
        Column("model_name", VARCHAR(100)),
        Column("training_start_time", TIMESTAMP()),
        Column("training_end_time", TIMESTAMP()),
        Column("total_training_time", Integer),
        Column("training_data_start_date", TIMESTAMP()),
        Column("training_data_end_date", TIMESTAMP()),
        Column("number_of_dump_data", Integer),
        Column("number_of_training_data", Integer),
        Column("number_of_positive_samples_in_training_data", Integer),
        Column("number_of_negative_samples_in_training_data", Integer),
        Column("number_of_validation_data", Integer),
        Column("true_label_column", VARCHAR(45)),
        Column("number_of_positive_samples_in_validation_data", Integer),
        Column("number_of_negative_samples_in_validation_data", Integer),
        Column("threshold", LONGTEXT),
        Column("tp", LONGTEXT),
        Column("fp", LONGTEXT),
        Column("tn", LONGTEXT),
        Column("fn", LONGTEXT),
        Column("accuracy", LONGTEXT),
        Column("ppv", LONGTEXT),
        Column("recall", LONGTEXT),
        Column("f1_score", LONGTEXT),
        Column("fnr", LONGTEXT),
        Column("fpr", LONGTEXT),
        Column("npv", LONGTEXT),
        Column("fdr", LONGTEXT),
        Column("for_", LONGTEXT),
        Column("tnr", LONGTEXT),
        Column("auc", LONGTEXT),
        Column("reason", LONGTEXT),
        Column("return_code", LONGTEXT),
    )
    return base_table



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time:2020.09.12
# @author:xhrg
# @email:634789257@qq.com

import hashlib
import sys
import socket

version = sys.version_info.major

if version == 2:
    from .python_2x import *

if version == 3:
    from .python_3x import *

# 定义常量
CONFIGURATIONS = "configurations"
NOTIFICATION_ID = "notificationId"
NAMESPACE_NAME = "namespaceName"


# 对时间戳，uri，秘钥进行加签
def signature(timestamp, uri, secret):
    import hmac
    import base64
    string_to_sign = '' + timestamp + '\n' + uri
    hmac_code = hmac.new(secret.encode(), string_to_sign.encode(), hashlib.sha1).digest()
    return base64.b64encode(hmac_code).decode()


def url_encode_wrapper(params):
    return url_encode(params)


def no_key_cache_key(namespace, key):
    return "{}{}{}".format(namespace, len(namespace), key)


# 返回是否获取到的值，不存在则返回None
def get_value_from_dict(namespace_cache, key):
    if namespace_cache:
        kv_data = namespace_cache.get(CONFIGURATIONS)
        if kv_data is None:
            return None
        if key in kv_data:
            return kv_data[key]
    return None


def init_ip():
    try:
        s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
        s.connect(('8.8.8.8', 53))
        ip = s.getsockname()[0]
        return ip
    finally:
        s.close()
    return ""

def check_apollo_change(init_logger, event, namespace, key, value):
    if event == "delete":
        init_logger.info(f"delete '{namespace}' '{key}' '{value}'")
        pass
    elif event == "update":
        init_logger.info(f"update '{namespace}' '{key}' '{value}'")
        pass
    elif event == "add":
        init_logger.info(f"add '{namespace}' '{key}' '{value}'")
        pass


def get_apollo_value(client, key, namespace):
    value = client.get_value(key, namespace=namespace)
    if value:
        return value
    else:
        os.kill(0, 4)







import pandas as pd
import pickle
from sklearn import metrics
from ctypes import cdll, c_char_p
from sqlalchemy import create_engine
from sqlalchemy.orm import sessionmaker


return_code = {
    'pred_success': '4003',                 # 預測成功
    'pred_fail': '9905',                    # 預測失敗
    'container_start_success': '4006',      # 容器啟動成功
    'container_start_fail': '4008',         # 容器啟動失敗
    'train_success': '4007',                # 小模型訓練成功
    'train_all_success': '4008',            # 大模型訓練成功
    'train_fail': '9909',                   # 訓練失敗
    'train_save_fail': '9916',              # 知識庫儲存失敗
    'train_load_fail': '9913',              # 知識庫載入失敗
    'train_valid_fail': '9914',             # 驗證失敗
}


def decrypt_passwd(so_name, passwd):
    try:
        lib = cdll.LoadLibrary(so_name)
        lib.passedDecrypt.argtypes = [c_char_p]
        lib.passedDecrypt.restype = c_char_p
        password = lib.passedDecrypt(passwd.encode()).decode()
        return password
    except Exception:
        return passwd


def open_connection(db_uri):
    engine = create_engine(db_uri, echo=False)
    DBSession = sessionmaker(bind=engine)
    session = DBSession()
    return (session)


def close_connection(session):
    session.close()


class AITrainUtilsAPI:
    def save_variable(self, v, filename):
        f = open(filename, 'wb')
        pickle.dump(v, f)
        f.close()
        return filename

    def load_variavle(self, filename):
        f = open(filename, 'rb')
        r = pickle.load(f)
        f.close()
        return r

    def err_reason(self, e):
        error_class = e.__class__.__name__
        detail = e.args[0]
        errMsg = "[{}] {}".format(error_class, detail)
        return errMsg

    def report(self, return_code, accuracy=-1, precision=-1, recall=-1, f1=-1, reason="nan"):
        return {
            'return_code': return_code,
            "report": {
                'accuracy': accuracy,
                'precision': precision,
                'recall': recall,
                'f1_score': f1
            },
            "reason": reason
        }

    def confuse_classification_report(self, test_y, test_y_predicted):
        accuracy = round(metrics.accuracy_score(test_y, test_y_predicted), 2)
        precision = round(metrics.precision_score(test_y, test_y_predicted), 2)
        recall = round(metrics.recall_score(test_y, test_y_predicted), 2)
        f1 = round(metrics.f1_score(test_y, test_y_predicted), 2)
        return self.report("4007", accuracy, precision, recall, f1)

    def batch_read_csv_train(self, partial_fit_model, list_file_name, list_features, chunksize):
        self.DATA_TOTAL_ROW = 0
        self.VAL_POSITION_NUMBER = 0
        self.VAL_POSITION_NEGATIVE = 0
        
        self.DATA_TOTAL_ROW_VAL = 0
        epoch = 0
        for file in list_file_name:
            if "csv" in file or ".ipynb_checkpoints" in file:
                del list_file_name[list_file_name.index(file)]

        for file_name in list_file_name:
            df_generator = pd.read_json(self.file_path + file_name, lines=True, chunksize=chunksize)
            for df in df_generator:
                if epoch == 0:
                    self.df = df
                    self.DATA_TOTAL_ROW_VAL += len(self.df)
                    self.VAL_POSITION_NUMBER_VAL = list(self.df[self.true_lable_name]).count("N")
                    self.VAL_POSITION_NEGATIVE_VAL = list(self.df[self.true_lable_name]).count("Y")
                elif epoch == 1:
                    self.DATA_START_DATE = str(df["timestamp"].iloc[0])

                x = df.loc[:, list_features].values
                
                partial_fit_model.partial_fit(x)
                epoch += 1
                self.DATA_TOTAL_ROW += len(df)
                self.VAL_POSITION_NUMBER += list(df[self.true_lable_name]).count("N")
                self.VAL_POSITION_NEGATIVE += list(df[self.true_lable_name]).count("Y")

        self.VAL_POSITION_NUMBER = self.VAL_POSITION_NUMBER - self.VAL_POSITION_NUMBER_VAL
        self.VAL_POSITION_NEGATIVE = self.VAL_POSITION_NEGATIVE - self.VAL_POSITION_NEGATIVE_VAL
        self.DATA_TOTAL_ROW_TRAIN = self.DATA_TOTAL_ROW - self.DATA_TOTAL_ROW_VAL
        self.DATA_END_DARE = str(df["timestamp"].iloc[-1])
        self.df[self.true_lable_name] = self.df[self.true_lable_name].replace("N", 0)
        self.df[self.true_lable_name] = self.df[self.true_lable_name].replace("Y", 1)
        return partial_fit_model



from ctypes import c_char_p, POINTER, c_longlong, Structure, cdll


class GoSlice(Structure):
    _fields_ = [("data", POINTER(c_char_p)),
                ("len", c_longlong), ("cap", c_longlong)]


class UAInfo(Structure):
    _fields_ = [('DeviceType', c_char_p), ('DeviceModel', c_char_p), ('DeviceBrand', c_char_p),
                ('CPUArch', c_char_p), ('OSName',
                                        c_char_p), ('OSVersion', c_char_p),
                ('BrowserName', c_char_p), ('BrowserVersion',
                                            c_char_p), ('BrowserEngine', c_char_p)
                ]


def call_so(lib_path="../include/ua.so"):
    lib_ua = cdll.LoadLibrary(lib_path)
    return lib_ua


class UserAgentDecoder:
    def __init__(self, lib_path):
        self.lib_ua = self.read_so(lib_path=lib_path)

    def read_so(self, lib_path):
        lib_ua = cdll.LoadLibrary(lib_path)
        return lib_ua

    def run(self, ua, width, height, ratio, platform, gpuName):
        self.lib_ua.UADetection.argtypes = [GoSlice]
        self.lib_ua.UADetection.restype = UAInfo

        ua_result = self.lib_ua.UADetection(GoSlice((c_char_p * 6)(
            ua.encode(),
            width.encode(),
            height.encode(),
            ratio.encode(),
            platform.encode(),
            gpuName.encode()
        ), 6, 6))
        dict_ua = {
            "DeviceType": ua_result.DeviceType.decode(),
            "DeviceModel": ua_result.DeviceModel.decode(),
            "DeviceBrand": ua_result.DeviceBrand.decode(),
            "OSName": ua_result.OSName.decode(),
            "OSVersion": ua_result.OSVersion.decode(),
            "Browser": ua_result.BrowserName.decode(),
            "BrowserVersion": ua_result.BrowserVersion.decode(),
            "CPUArch": ua_result.CPUArch.decode(),
            "BrowserEngine": ua_result.BrowserEngine.decode(),
        }
        return dict_ua


def main():
    ua = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_14_6) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/80.0.3987.158 Safari/537.36"
    width = "1680"
    height = "1050"
    ratio = "2"
    platform = "iPad"
    gpuName = "Apple GPU"
    uad = UserAgentDecoder("../../Gitlab/hitrustai/Module/useragent/ua.so")
    dict_ua = uad.run(ua, width, height, ratio, platform, gpuName)
    print(dict_ua)


if __name__ == '__main__':
    main()



import pandas as pd
import pymysql
import numpy as np
from tqdm import tqdm


class DBDownload:
    def __init__(
        self, host="192.168.10.201",
        user="test", passwd="13572468",
        db_name="diiarelease",
        port="3305",
        table="tb_fraud_detect_predict_result",
        batch_size=10000,
        file_name_csv="model_predict.csv",
        sql_time=""
    ):
        self.host = host
        self.user = user
        self.passwd = passwd
        self.db_name = db_name
        self.port = port
        self.table = table
        self.batch_size = batch_size
        self.file_name_csv = file_name_csv
        self.sql_time = sql_time

    def get_db_need_looptime(self, batch=100000):
        con = pymysql.connect(
            db=self.db_name,
            user=self.user,
            passwd=self.passwd,
            host=self.host,
            port=self.port
        )
        
        sql_cmd = """
        SELECT count(pk_id) FROM %s %s;
        """ % (self.table, self.sql_time)
        cursor = con.cursor()
        cursor.execute(sql_cmd)
        data = cursor.fetchone()
        con.close()
        return data[0] // batch + 1

    def run(self, sql_cmd):
        con = pymysql.connect(
            db=self.db_name,
            user=self.user,
            passwd=self.passwd,
            host=self.host,
            port=self.port
        )
        pk_id = 0
        loop_times = self.get_db_need_looptime(self.batch_size)
        for i in tqdm(range(loop_times)):
            print("pk_id:", pk_id)
            sql_time = ""
            if self.sql_time == "":
                sql_time = "where "
            else:
                sql_time = self.sql_time + "and"

            sql = """%s pk_id > %d limit %d;""" % (sql_time, pk_id, self.batch_size)
            df = pd.read_sql(sql_cmd + sql, con)
            print("df長度:", len(df))
            
            
            if pk_id == 0:
                df.to_csv(self.file_name_csv, index=False)
            else:
                df.to_csv(self.file_name_csv, mode="a", header=False, index=False)
            pk_id = list(df["pk_id"][-1:])[0]


def main():
    db_name = "diia_release"
    table = "deviceinfo"
    diia = DBDownload(
        db_name=db_name,
        user="root",
        passwd="root16313302",
        host="192.168.10.112",
        port=3305,
        table=table,
        batch_size=10000,
        file_name_csv="new_diia.csv",
        sql_time="where udid like '154637530395207*%'"
    )
    sql_cmd = """
        SELECT * FROM %s.%s
    """ % (db_name, table)
    diia.run(sql_cmd)
    print("---成功---")


if __name__ == '__main__':
    main()



from sklearn import datasets
from sklearn.model_selection import train_test_split
from sklearn.linear_model import LogisticRegression
from sklearn.metrics import auc
import numpy as np
import pandas as pd


class ModelPerfornance:
    def __init__(self, score_type="total_score"):
        self.score_type = score_type

    # def calculate_prc_auc(self, recall_lst, precision_lst):
    #     try:
    #         pr_list = list(zip(precision_lst, recall_lst))
    #         pr_list = [item for item in pr_list if -1 not in item]
    #         precision_lst, recall_lst = list(zip(*pr_list))
    #         auc_precision_recall = round(auc(recall_lst, precision_lst), 3)
    #     except Exception:
    #         precision_lst[-1] = -0.99999
    #         precision_lst[-2] = -0.99999
    #         pr_list = list(zip(precision_lst, recall_lst))
    #         pr_list = [item for item in pr_list if -1 not in item]
    #         precision_lst, recall_lst = list(zip(*pr_list))
    #         auc_precision_recall = round(auc(recall_lst, precision_lst), 3)
    #     return auc_precision_recall

    def calculate_prc_auc(self, recall_lst, precision_lst):
        pr_list = list(zip(precision_lst, recall_lst))
        pr_list = [item for item in pr_list if -1 not in item]
        if len(pr_list):
            precision_lst, recall_lst = list(zip(*pr_list))
            auc_precision_recall = round(auc(recall_lst, precision_lst), 3)
            return [auc_precision_recall]
        return [-1]

    def division(self, a, b):
        if sum(b):
            x = (a/b).round(3)
            return np.nan_to_num(x, nan=-1).tolist()
        return [-1]*a.shape[0]

    def filter_by_score_type(self, data):
        if self.score_type == 'policy_score':
            return data[::-1]
        return data

    def performance_output(self, y_test, y_score):
        threshold_lst = []
        tp_lst = fp_lst = tn_lst = fn_lst = np.array([])

        if len(y_score) and len(y_test):
            for i in range(0, 11, 1):
                threshold = i / 10
                y_pre = np.where(y_score >= threshold, 1, 0)

                if self.score_type == 'policy_score':
                    threshold_lst.append(round(-2*threshold+1, 3))
                elif self.score_type == 'total_score':
                    threshold_lst.append(threshold)

                tb = pd.DataFrame({'predict_label': y_pre, 'true_label': y_test})
                tp = sum(((tb.true_label == 1) & (tb.predict_label == 1)))
                fp = sum(((tb.true_label == 0) & (tb.predict_label == 1)))
                tn = sum(((tb.true_label == 0) & (tb.predict_label == 0)))
                fn = sum(((tb.true_label == 1) & (tb.predict_label == 0)))

                tp_lst = np.append(tp_lst, tp)
                fp_lst = np.append(fp_lst, fp)
                tn_lst = np.append(tn_lst, tn)
                fn_lst = np.append(fn_lst, fn)

            accuracy_lst = self.division(tp_lst+tn_lst, tp_lst+tn_lst+fp_lst+fn_lst)
            precision_lst = self.division(tp_lst, tp_lst+fp_lst)
            recall_lst = self.division(tp_lst, tp_lst+fn_lst)
            f1_score_lst = self.division(
                2*np.array(precision_lst)*np.array(recall_lst),
                np.array(precision_lst)+np.array(recall_lst)
            )
            fnr_lst = self.division(fn_lst, tp_lst+fn_lst)
            fpr_lst = self.division(fp_lst, fp_lst+tn_lst)
            npv_lst = self.division(tn_lst, fn_lst+tn_lst)
            fdr_lst = self.division(fp_lst, tp_lst+fp_lst)
            for_lst = self.division(fn_lst, fn_lst+tn_lst)
            tnr_lst = self.division(tn_lst, fp_lst+tn_lst)

            auc_precision_recall = self.calculate_prc_auc(fpr_lst, recall_lst)
        else:
            accuracy_lst = precision_lst = recall_lst = f1_score_lst = []
            fnr_lst = fpr_lst = npv_lst = fdr_lst = for_lst = tnr_lst = []
            auc_precision_recall = [-1]

        result = {
            "threshold_lst": self.filter_by_score_type(threshold_lst),
            "tp_lst": self.filter_by_score_type(tp_lst.tolist()),
            "fp_lst": self.filter_by_score_type(fp_lst.tolist()),
            "tn_lst": self.filter_by_score_type(tn_lst.tolist()),
            "fn_lst": self.filter_by_score_type(fn_lst.tolist()),
            "accuracy_lst": self.filter_by_score_type(accuracy_lst),
            "precision_lst": self.filter_by_score_type(precision_lst),
            "recall_lst": self.filter_by_score_type(recall_lst),
            "f1_score_lst": self.filter_by_score_type(f1_score_lst),
            "fnr_lst": self.filter_by_score_type(fnr_lst),
            "fpr_lst": self.filter_by_score_type(fpr_lst),
            "npv_lst": self.filter_by_score_type(npv_lst),
            "fdr_lst": self.filter_by_score_type(fdr_lst),
            "for_lst": self.filter_by_score_type(for_lst),
            "tnr_lst": self.filter_by_score_type(tnr_lst),
            "auc_lst": self.filter_by_score_type(auc_precision_recall)
        }
        return result

    
    def model_train(self):
        random_state = 1
        # Create dataset for binary classification with 5 predictors
        X, y = datasets.make_classification(n_samples=1000,
                                            n_features=5,
                                            n_informative=3,
                                            n_redundant=2)
        # n_redundant=2, random_state=random_state)

        # Split into training and test
        X_train, X_test, y_train, y_test = train_test_split(X, y,
                                                            test_size=0.5)
        # test_size=0.5 ,random_state=random_state)

        # Create classifier using logistic regression
        classifier = LogisticRegression(random_state=random_state)
        classifier.fit(X_train, y_train)
        y_score = classifier.predict_proba(X_test)[:, 1]
        # result = self.performance_output(y_test, y_score)
        # return result
        return y_test, y_score


def main():
    mp = ModelPerfornance(score_type='policy_score')
    list_y_test, list_y_score = mp.model_train()
    result = mp.performance_output(list_y_test, list_y_score)
    print(result)


if __name__ == "__main__":
    main()



from sqlalchemy import Column, Integer, String, text, DateTime, Numeric, JSON, BOOLEAN, Text
from sqlalchemy.dialects.mysql import TIMESTAMP
from sqlalchemy.ext.declarative import declarative_base

'''
Scripts to create a table:

# from sqlalchemy import create_engine  

# DBname = 'acqfd_test'
# db_string = f"mysql+pymysql://isrfd:isrfd16313302@35.229.153.69/{DBname}?charset=utf8mb4&binary_prefix=true"
# engine = create_engine(db_string, echo=True)  
# Base = declarative_base() 

# yourtable

# Base.metadata.create_all(engine)  

'''


Base = declarative_base()
# metadata = Base.metadata


class ThreeDS1Detect(Base):
    __tablename__ = 'tb_3ds1_detect'
    __table_args__ = {'comment': '原始3DS驗證表(3DS1)'}

    pk_id = Column(Integer, primary_key=True,
                   autoincrement=True, nullable=False)
    client_info_id = Column(String(100, 'utf8mb4_unicode_ci'))
    sn = Column(String(60, 'utf8mb4_unicode_ci'), nullable=False)
    customer_servertime = Column(TIMESTAMP(fsp=6), nullable=False)
    sp_tx_id = Column(String(45, 'utf8mb4_unicode_ci'), nullable=False)
    pan_hash = Column(String(128, 'utf8mb4_unicode_ci'), nullable=False)
    pan_expire_year = Column(Integer, nullable=False)
    pan_expire_month = Column(Integer, nullable=False)
    acq_bin = Column(String(45, 'utf8mb4_unicode_ci'), nullable=False)
    country_code = Column(String(45, 'utf8mb4_unicode_ci'), nullable=False)
    merchant_id = Column(String(45, 'utf8mb4_unicode_ci'), nullable=False)
    merchant_name = Column(String(45, 'utf8mb4_unicode_ci'), nullable=False)
    merchant_url = Column(String(45, 'utf8mb4_unicode_ci'), nullable=False)
    purchase_amount = Column(Numeric(16, 5), nullable=False)
    purchase_currency = Column(
        String(45, 'utf8mb4_unicode_ci'), nullable=False)
    rtn_url = Column(Text, comment='付款結果回傳網址')
    threeds_type = Column(Integer, nullable=False)
    # return_code       = Column(String(10, 'utf8mb4_unicode_ci'), nullable=False)
    create_time = Column(TIMESTAMP(fsp=6), nullable=False,
                         server_default=text("CURRENT_TIMESTAMP(6)"))

    def __init__(self, **kwargs):
        for property, value in kwargs.items():
            setattr(self, property, value)


class ThreeDS2Detect(Base):
    __tablename__ = 'tb_3ds2_detect'
    __table_args__ = {'comment': '原始3DS驗證表(3DS2)'}

    pk_id = Column(Integer, primary_key=True,
                   autoincrement=True, nullable=False)
    client_info_id = Column(String(100, 'utf8mb4_unicode_ci'))
    sn = Column(String(45, 'utf8mb4_unicode_ci'), nullable=False)
    customer_servertime = Column(TIMESTAMP(fsp=6), nullable=False)
    sp_tx_id = Column(String(45, 'utf8mb4_unicode_ci'), nullable=False)

    # R
    threeds_comp_ind = Column(String(1, 'utf8mb4_unicode_ci'))
    threeds_req_auth_ind = Column(String(2, 'utf8mb4_unicode_ci'))
    threeds_req_id = Column(String(35, 'utf8mb4_unicode_ci'))
    threeds_req_name = Column(String(40, 'utf8mb4_unicode_ci'))
    threeds_req_url = Column(String(1024, 'utf8mb4_unicode_ci'))
    acq_bin = Column(String(11, 'utf8mb4_unicode_ci'))
    acq_merid = Column(String(35, 'utf8mb4_unicode_ci'))
    browser_accept_header = Column(String(2048, 'utf8mb4_unicode_ci'))
    browser_java_enabled = Column(String(8))
    browser_language = Column(String(8, 'utf8mb4_unicode_ci'))
    browser_color_depth = Column(String(2, 'utf8mb4_unicode_ci'))
    browser_screen_height = Column(String(6, 'utf8mb4_unicode_ci'))
    browser_screen_width = Column(String(6, 'utf8mb4_unicode_ci'))
    browser_tz = Column(String(5, 'utf8mb4_unicode_ci'))
    browser_user_agent = Column(String(2048, 'utf8mb4_unicode_ci'))
    card_expiry_date = Column(String(4, 'utf8mb4_unicode_ci'))
    acct_number = Column(String(128, 'utf8mb4_unicode_ci'))
    device_channel = Column(String(2, 'utf8mb4_unicode_ci'))
    device_render_options = Column(JSON)
    mcc = Column(String(4, 'utf8mb4_unicode_ci'))
    mer_country_code = Column(String(3, 'utf8mb4_unicode_ci'))
    mer_name = Column(String(40, 'utf8mb4_unicode_ci'))
    msg_category = Column(String(2, 'utf8mb4_unicode_ci'))
    msg_type = Column(String(4, 'utf8mb4_unicode_ci'))
    msg_version = Column(String(8, 'utf8mb4_unicode_ci'))
    purchase_amount = Column(String(48, 'utf8mb4_unicode_ci'))
    purchase_currency = Column(String(3, 'utf8mb4_unicode_ci'))
    purchase_exponent = Column(String(1, 'utf8mb4_unicode_ci'))
    purchase_date = Column(String(14, 'utf8mb4_unicode_ci'))
    sdk_appid = Column(String(36, 'utf8mb4_unicode_ci'))
    sdk_enc_data = Column(String(9000, 'utf8mb4_unicode_ci'))
    sdk_ephem_pub_key = Column(String(256, 'utf8mb4_unicode_ci'))
    sdk_max_timeout = Column(String(2, 'utf8mb4_unicode_ci'))
    sdk_ref_number = Column(String(36, 'utf8mb4_unicode_ci'))
    sdk_transid = Column(String(36, 'utf8mb4_unicode_ci'))
    notification_url = Column(String(256, 'utf8mb4_unicode_ci'))
    three_ri_ind = Column(String(2, 'utf8mb4_unicode_ci'))

    # O
    threeds_req_auth_info = Column(JSON)
    threeds_req_challenge_ind = Column(String(2, 'utf8mb4_unicode_ci'))
    threeds_req_prior_auth_info = Column(JSON)
    acct_type = Column(String(2, 'utf8mb4_unicode_ci'))
    addr_match = Column(String(1, 'utf8mb4_unicode_ci'))
    browser_ip = Column(String(45, 'utf8mb4_unicode_ci'))
    acct_info = Column(JSON)
    acct_id = Column(String(64, 'utf8mb4_unicode_ci'))
    bill_addr_city = Column(String(50, 'utf8mb4_unicode_ci'))
    bill_addr_country = Column(String(3, 'utf8mb4_unicode_ci'))
    bill_addr_line1 = Column(String(50, 'utf8mb4_unicode_ci'))
    bill_addr_line2 = Column(String(50, 'utf8mb4_unicode_ci'))
    bill_addr_line3 = Column(String(50, 'utf8mb4_unicode_ci'))
    bill_addr_postcode = Column(String(16, 'utf8mb4_unicode_ci'))
    bill_addr_state = Column(String(3, 'utf8mb4_unicode_ci'))
    email = Column(String(254, 'utf8mb4_unicode_ci'))
    home_phone = Column(JSON)
    mobile_phone = Column(JSON)
    cardholder_name = Column(String(45, 'utf8mb4_unicode_ci'))
    ship_addr_city = Column(String(50, 'utf8mb4_unicode_ci'))
    ship_addr_country = Column(String(3, 'utf8mb4_unicode_ci'))
    ship_addr_line1 = Column(String(50, 'utf8mb4_unicode_ci'))
    ship_addr_line2 = Column(String(50, 'utf8mb4_unicode_ci'))
    ship_addr_line3 = Column(String(50, 'utf8mb4_unicode_ci'))
    ship_addr_postcode = Column(String(16, 'utf8mb4_unicode_ci'))
    ship_addr_state = Column(String(3, 'utf8mb4_unicode_ci'))
    work_phone = Column(JSON)
    mer_risk_indicator = Column(JSON)
    pmt_transid = Column(String(36, 'utf8mb4_unicode_ci'))
    card_scheme = Column(String(1, 'utf8mb4_unicode_ci'))

    # C
    threeds_server_transid = Column(String(36, 'utf8mb4_unicode_ci'))
    pay_token_ind = Column(String(4, 'utf8mb4_unicode_ci'))
    purchase_instal_data = Column(String(3, 'utf8mb4_unicode_ci'))
    recurring_expiry = Column(String(8, 'utf8mb4_unicode_ci'))
    recurring_freq = Column(String(4, 'utf8mb4_unicode_ci'))
    trans_type = Column(String(2, 'utf8mb4_unicode_ci'))

    # msg_extension               = Column(String(8, 'utf8mb4_unicode_ci'))
    threeds_type = Column(Integer, nullable=False)
    rtn_url = Column(Text, comment='付款結果回傳網址')
    # pan_expire_year   = Column(Integer, nullable=False)
    # pan_expire_month  = Column(Integer, nullable=False)
    create_time = Column(TIMESTAMP(fsp=6), nullable=False,
                         server_default=text("CURRENT_TIMESTAMP(6)"))

    def __init__(self, **kwargs):
        for property, value in kwargs.items():

            setattr(self, property, value)


class ThreeDS1DetectCleaned(Base):
    __tablename__ = 'tb_3ds1_detect_cleaned'
    __table_args__ = {'comment': '3DS1驗證資料表'}

    pk_id = Column(Integer, primary_key=True,
                   autoincrement=True, nullable=False)
    client_info_id = Column(String(100, 'utf8mb4_unicode_ci'))
    sn = Column(String(60, 'utf8mb4_unicode_ci'), nullable=False)
    customer_servertime = Column(TIMESTAMP(fsp=6), nullable=False)
    sp_tx_id = Column(String(45, 'utf8mb4_unicode_ci'), nullable=False)
    hashcard = Column(String(128, 'utf8mb4_unicode_ci'), nullable=False)
    pan_expire_year = Column(Integer, nullable=False)
    pan_expire_month = Column(Integer, nullable=False)
    acq_bin = Column(Integer, nullable=False)
    country_code = Column(String(45, 'utf8mb4_unicode_ci'), nullable=False)
    merchant_id = Column(String(45, 'utf8mb4_unicode_ci'), nullable=False)
    merchant_name = Column(String(45, 'utf8mb4_unicode_ci'), nullable=False)
    merchant_url = Column(String(45, 'utf8mb4_unicode_ci'), nullable=False)
    purchase_amount = Column(Numeric(16, 5), nullable=False)
    purchase_currency = Column(
        String(45, 'utf8mb4_unicode_ci'), nullable=False)
    threeds_type = Column(Integer, nullable=False)
    # return_code       = Column(String(10, 'utf8mb4_unicode_ci'), nullable=False)
    create_time = Column(TIMESTAMP(fsp=6), nullable=False,
                         server_default=text("CURRENT_TIMESTAMP(6)"))


class ThreeDS2DetectCleaned(Base):
    __tablename__ = 'tb_3ds2_detect_cleaned'
    __table_args__ = {'comment': '3DS2驗證資料表'}

    pk_id = Column(Integer, primary_key=True,
                   autoincrement=True, nullable=False)
    client_info_id = Column(String(100, 'utf8mb4_unicode_ci'))
    sn = Column(String(45, 'utf8mb4_unicode_ci'), nullable=False)
    customer_servertime = Column(TIMESTAMP(fsp=6), nullable=False)
    sp_tx_id = Column(String(45, 'utf8mb4_unicode_ci'), nullable=False)

    # R
    threeds_comp_ind = Column(String(1, 'utf8mb4_unicode_ci'))
    threeds_req_auth_ind = Column(Integer, nullable=False)
    threeds_req_id = Column(String(35, 'utf8mb4_unicode_ci'))
    threeds_req_name = Column(String(40, 'utf8mb4_unicode_ci'))
    threeds_req_url = Column(String(1024, 'utf8mb4_unicode_ci'))
    acq_bin = Column(String(11, 'utf8mb4_unicode_ci'))
    acq_merid = Column(String(35, 'utf8mb4_unicode_ci'))
    browser_accept_header = Column(String(2048, 'utf8mb4_unicode_ci'))
    browser_java_enabled = Column(Integer, nullable=False)
    browser_language = Column(String(8, 'utf8mb4_unicode_ci'))
    browser_color_depth = Column(Integer, nullable=False)
    browser_screen_height = Column(Integer, nullable=False)
    browser_screen_width = Column(Integer, nullable=False)
    browser_tz = Column(Integer, nullable=False)
    browser_user_agent = Column(String(2048, 'utf8mb4_unicode_ci'))
    # card_expiry_date
    hashcard = Column(String(128, 'utf8mb4_unicode_ci'))
    device_channel = Column(Integer, nullable=False)
    device_render_options = Column(JSON)
    mcc = Column(Integer, nullable=False)
    mer_country_code = Column(String(3, 'utf8mb4_unicode_ci'))
    mer_name = Column(String(40, 'utf8mb4_unicode_ci'))
    msg_category = Column(Integer, nullable=False)
    msg_type = Column(String(4, 'utf8mb4_unicode_ci'))
    msg_version = Column(String(8, 'utf8mb4_unicode_ci'))
    purchase_amount = Column(Numeric(16, 5), nullable=False)
    purchase_currency = Column(Integer, nullable=False)
    purchase_exponent = Column(Integer, nullable=False)
    purchase_date = Column(DateTime, nullable=False)
    sdk_appid = Column(String(36, 'utf8mb4_unicode_ci'))
    sdk_enc_data = Column(String(9000, 'utf8mb4_unicode_ci'))
    sdk_ephem_pub_key = Column(String(256, 'utf8mb4_unicode_ci'))
    sdk_max_timeout = Column(String(2, 'utf8mb4_unicode_ci'))
    sdk_ref_number = Column(String(36, 'utf8mb4_unicode_ci'))
    sdk_transid = Column(String(36, 'utf8mb4_unicode_ci'))
    notification_url = Column(String(256, 'utf8mb4_unicode_ci'))
    three_ri_ind = Column(String(2, 'utf8mb4_unicode_ci'))

    # O
    threeds_req_auth_info = Column(JSON)
    threeds_req_challenge_ind = Column(Integer, nullable=False)
    threeds_req_prior_auth_info = Column(JSON)
    acct_type = Column(Integer, nullable=False)
    addr_match = Column(String(1, 'utf8mb4_unicode_ci'))
    browser_ip = Column(String(45, 'utf8mb4_unicode_ci'))
    acct_info = Column(JSON)
    acct_id = Column(String(64, 'utf8mb4_unicode_ci'))
    bill_addr_city = Column(String(50, 'utf8mb4_unicode_ci'))
    bill_addr_country = Column(String(3, 'utf8mb4_unicode_ci'))
    bill_addr_line1 = Column(String(50, 'utf8mb4_unicode_ci'))
    bill_addr_line2 = Column(String(50, 'utf8mb4_unicode_ci'))
    bill_addr_line3 = Column(String(50, 'utf8mb4_unicode_ci'))
    bill_addr_postcode = Column(String(16, 'utf8mb4_unicode_ci'))
    bill_addr_state = Column(String(3, 'utf8mb4_unicode_ci'))
    email = Column(String(254, 'utf8mb4_unicode_ci'))
    home_phone = Column(JSON)
    mobile_phone = Column(JSON)
    cardholder_name = Column(String(45, 'utf8mb4_unicode_ci'))
    ship_addr_city = Column(String(50, 'utf8mb4_unicode_ci'))
    ship_addr_country = Column(String(3, 'utf8mb4_unicode_ci'))
    ship_addr_line1 = Column(String(50, 'utf8mb4_unicode_ci'))
    ship_addr_line2 = Column(String(50, 'utf8mb4_unicode_ci'))
    ship_addr_line3 = Column(String(50, 'utf8mb4_unicode_ci'))
    ship_addr_postcode = Column(String(16, 'utf8mb4_unicode_ci'))
    ship_addr_state = Column(String(3, 'utf8mb4_unicode_ci'))
    work_phone = Column(JSON)
    mer_risk_indicator = Column(JSON)
    pmt_transid = Column(String(36, 'utf8mb4_unicode_ci'))
    card_scheme = Column(String(1, 'utf8mb4_unicode_ci'))

    # C
    threeds_server_transid = Column(String(36, 'utf8mb4_unicode_ci'))
    pay_token_ind = Column(BOOLEAN)
    purchase_instal_data = Column(String(3, 'utf8mb4_unicode_ci'))
    recurring_expiry = Column(String(8, 'utf8mb4_unicode_ci'))
    recurring_freq = Column(String(4, 'utf8mb4_unicode_ci'))
    trans_type = Column(Integer, nullable=False)

    # msg_extension               = Column(String(8, 'utf8mb4_unicode_ci'))
    threeds_type = Column(Integer, nullable=False)
    pan_expire_year = Column(Integer, nullable=False)
    pan_expire_month = Column(Integer, nullable=False)
    create_time = Column(TIMESTAMP(fsp=6), nullable=False,
                         server_default=text("CURRENT_TIMESTAMP(6)"))


class ThreeDS1DetectAR(Base):
    __tablename__ = 'tb_3ds1_detect_ar'
    __table_args__ = {'comment': '3DS1預測AR表'}

    pk_id = Column(Integer, primary_key=True,
                   autoincrement=True, nullable=False)
    client_info_id = Column(String(100, 'utf8mb4_unicode_ci'))
    sn = Column(String(60, 'utf8mb4_unicode_ci'), nullable=False)
    hashcard = Column(String(128, 'utf8mb4_unicode_ci'), nullable=False)
    timestamp = Column(DateTime, nullable=False)
    # cardcount         = Column(Integer, nullable=False)
    # servertime        = Column(DateTime, nullable=False)
    sp_tx_id = Column(Numeric(8, 5), nullable=False)
    # pan_hash          = Column(String(128, 'utf8mb4_unicode_ci'), nullable=False)
    pan_expire_year = Column(Numeric(8, 5), nullable=False)
    pan_expire_month = Column(Numeric(8, 5), nullable=False)
    acq_bin = Column(Numeric(8, 5), nullable=False)
    country_code = Column(Numeric(8, 5), nullable=False)
    merchant_id = Column(Numeric(8, 5), nullable=False)
    merchant_name = Column(Numeric(8, 5), nullable=False)
    merchant_url = Column(Numeric(8, 5), nullable=False)
    purchase_amount = Column(Numeric(8, 5), nullable=False)
    purchase_currency = Column(Numeric(8, 5), nullable=False)
    # threeds_type      = Column(Integer, nullable=False)
    create_time = Column(TIMESTAMP(fsp=6), nullable=False,
                         server_default=text("CURRENT_TIMESTAMP(6)"))


class ThreeDS1DetectAI(Base):
    __tablename__ = 'tb_temp_3ds1_detect_ai'
    __table_args__ = {'comment': '3DS1預測AI表'}

    pk_id = Column(Integer, primary_key=True,
                   autoincrement=True, nullable=False)
    client_info_id = Column(String(100, 'utf8mb4_unicode_ci'))
    sn = Column(String(60, 'utf8mb4_unicode_ci'), nullable=False)
    hashcard = Column(String(128, 'utf8mb4_unicode_ci'), nullable=False)
    timestamp = Column(DateTime, nullable=False)
    # pan_hash          = Column(String(128, 'utf8mb4_unicode_ci'), nullable=False)
    # servertime        = Column(DateTime, nullable=False)
    # is_new            = Column(String(5, 'utf8mb4_unicode_ci'), nullable=False)
    ai = Column(Numeric(8, 5), nullable=False)
    cl = Column(Numeric(8, 5), nullable=False)
    ucl = Column(Numeric(8, 5), nullable=False)
    is_abnormal = Column(Integer, nullable=False)
    create_time = Column(TIMESTAMP(fsp=6), nullable=False,
                         server_default=text("CURRENT_TIMESTAMP(6)"))


class ThreeDS2DetectAR(Base):
    __tablename__ = 'tb_3ds2_detect_ar'
    __table_args__ = {'comment': '3DS2預測AR表'}

    pk_id = Column(Integer, primary_key=True,
                   autoincrement=True, nullable=False)
    client_info_id = Column(String(100, 'utf8mb4_unicode_ci'))
    sn = Column(String(45, 'utf8mb4_unicode_ci'), nullable=False)
    hashcard = Column(String(128, 'utf8mb4_unicode_ci'), nullable=False)
    timestamp = Column(DateTime, nullable=False)
    # servertime                  = Column(DateTime, nullable=False)
    sp_tx_id = Column(Numeric(8, 5))

    # R
    threeds_comp_ind = Column(Numeric(8, 5))
    threeds_req_auth_ind = Column(Numeric(8, 5))
    threeds_req_id = Column(Numeric(8, 5))
    threeds_req_name = Column(Numeric(8, 5))
    threeds_req_url = Column(Numeric(8, 5))
    acq_bin = Column(Numeric(8, 5))
    acq_merid = Column(Numeric(8, 5))
    browser_accept_header = Column(Numeric(8, 5))
    browser_java_enabled = Column(Numeric(8, 5))
    browser_language = Column(Numeric(8, 5))
    browser_color_depth = Column(Numeric(8, 5))
    browser_screen_height = Column(Numeric(8, 5))
    browser_screen_width = Column(Numeric(8, 5))
    browser_tz = Column(Numeric(8, 5))
    browser_user_agent = Column(Numeric(8, 5))
    acct_number = Column(Numeric(8, 5))
    device_channel = Column(Numeric(8, 5))
    device_render_options = Column(Numeric(8, 5))
    mcc = Column(Numeric(8, 5))
    mer_country_code = Column(Numeric(8, 5))
    mer_name = Column(Numeric(8, 5))
    msg_category = Column(Numeric(8, 5))
    msg_type = Column(Numeric(8, 5))
    msg_version = Column(Numeric(8, 5))
    purchase_amount = Column(Numeric(8, 5))
    purchase_currency = Column(Numeric(8, 5))
    purchase_exponent = Column(Numeric(8, 5))
    purchase_date = Column(Numeric(8, 5))
    sdk_appid = Column(Numeric(8, 5))
    sdk_enc_data = Column(Numeric(8, 5))
    sdk_ephem_pub_key = Column(Numeric(8, 5))
    sdk_max_timeout = Column(Numeric(8, 5))
    sdk_ref_number = Column(Numeric(8, 5))
    sdk_transid = Column(Numeric(8, 5))
    notification_url = Column(Numeric(8, 5))
    three_ri_ind = Column(Numeric(8, 5))

    # O
    threeds_req_auth_info = Column(Numeric(8, 5))
    threeds_req_challenge_ind = Column(Numeric(8, 5))
    threeds_req_prior_auth_info = Column(Numeric(8, 5))
    acct_type = Column(Numeric(8, 5))
    addr_match = Column(Numeric(8, 5))
    browser_ip = Column(Numeric(8, 5))
    acct_info = Column(Numeric(8, 5))
    acct_id = Column(Numeric(8, 5))
    bill_addr_city = Column(Numeric(8, 5))
    bill_addr_country = Column(Numeric(8, 5))
    bill_addr_line1 = Column(Numeric(8, 5))
    bill_addr_line2 = Column(Numeric(8, 5))
    bill_addr_line3 = Column(Numeric(8, 5))
    bill_addr_postcode = Column(Numeric(8, 5))
    bill_addr_state = Column(Numeric(8, 5))
    email = Column(Numeric(8, 5))
    home_phone = Column(Numeric(8, 5))
    mobile_phone = Column(Numeric(8, 5))
    cardholder_name = Column(Numeric(8, 5))
    ship_addr_city = Column(Numeric(8, 5))
    ship_addr_country = Column(Numeric(8, 5))
    ship_addr_line1 = Column(Numeric(8, 5))
    ship_addr_line2 = Column(Numeric(8, 5))
    ship_addr_line3 = Column(Numeric(8, 5))
    ship_addr_postcode = Column(Numeric(8, 5))
    ship_addr_state = Column(Numeric(8, 5))
    work_phone = Column(Numeric(8, 5))
    mer_risk_indicator = Column(Numeric(8, 5))
    pmt_transid = Column(Numeric(8, 5))
    card_scheme = Column(Numeric(8, 5))

    # C
    threeds_server_transid = Column(Numeric(8, 5))
    pay_token_ind = Column(Numeric(8, 5))
    purchase_instal_data = Column(Numeric(8, 5))
    recurring_expiry = Column(Numeric(8, 5))
    recurring_freq = Column(Numeric(8, 5))
    trans_type = Column(Numeric(8, 5))

    pan_expire_year = Column(Numeric(8, 5))
    pan_expire_month = Column(Numeric(8, 5))
    create_time = Column(TIMESTAMP(fsp=6), nullable=False,
                         server_default=text("CURRENT_TIMESTAMP(6)"))


class ThreeDS2DetectAI(Base):
    __tablename__ = 'tb_temp_3ds2_detect_ai'
    __table_args__ = {'comment': '3DS2預測AI表'}

    pk_id = Column(Integer, primary_key=True,
                   autoincrement=True, nullable=False)
    client_info_id = Column(String(100, 'utf8mb4_unicode_ci'))
    sn = Column(String(60, 'utf8mb4_unicode_ci'), nullable=False)
    hashcard = Column(String(128, 'utf8mb4_unicode_ci'), nullable=False)
    timestamp = Column(DateTime, nullable=False)
    # pan_hash          = Column(String(128, 'utf8mb4_unicode_ci'), nullable=False)
    # servertime        = Column(DateTime, nullable=False)
    # is_new            = Column(String(5, 'utf8mb4_unicode_ci'), nullable=False)
    ai = Column(Numeric(8, 5), nullable=False)
    cl = Column(Numeric(8, 5), nullable=False)
    ucl = Column(Numeric(8, 5), nullable=False)
    is_abnormal = Column(Integer, nullable=False)
    create_time = Column(TIMESTAMP(fsp=6), nullable=False,
                         server_default=text("CURRENT_TIMESTAMP(6)"))


class ThreeDs1RealResult(Base):
    __tablename__ = 'tb_3ds1_real_result'

    pk_id = Column(Integer, primary_key=True,
                   autoincrement=True, nullable=False)
    return_code = Column(String(10, 'utf8mb4_unicode_ci'))
    client_info_id = Column(String(100, 'utf8mb4_unicode_ci'))
    sn = Column(String(60, 'utf8mb4_unicode_ci'))
    customer_servertime = Column(TIMESTAMP(fsp=6), nullable=False)
    ve_status = Column(String(1, 'utf8mb4_unicode_ci'))
    pa_status = Column(String(1, 'utf8mb4_unicode_ci'))
    eci = Column(String(1, 'utf8mb4_unicode_ci'))
    error_code = Column(String(45, 'utf8mb4_unicode_ci'))
    error_msg = Column(String(45, 'utf8mb4_unicode_ci'))
    create_time = Column(TIMESTAMP(fsp=6), nullable=False,
                         server_default=text("CURRENT_TIMESTAMP(6)"))

    def __init__(self, **kwargs):
        for property, value in kwargs.items():

            setattr(self, property, value)


class ThreeDs1RealResultBacklog(Base):
    __tablename__ = 'tb_3ds1_real_result_backlog'

    pk_id = Column(Integer, primary_key=True,
                   autoincrement=True, nullable=False)
    sn = Column(String(45, 'utf8mb4_unicode_ci'))

    return_code = Column(String(10, 'utf8mb4_unicode_ci'))
    create_time = Column(TIMESTAMP(fsp=6), nullable=False,
                         server_default=text("CURRENT_TIMESTAMP(6)"))


class ThreeDs2RealResult(Base):
    __tablename__ = 'tb_3ds2_real_result'

    pk_id = Column(Integer, primary_key=True,
                   autoincrement=True, nullable=False)
    return_code = Column(String(10, 'utf8mb4_unicode_ci'))
    client_info_id = Column(String(100, 'utf8mb4_unicode_ci'))
    sn = Column(String(60, 'utf8mb4_unicode_ci'))
    customer_servertime = Column(TIMESTAMP(fsp=6), nullable=False)
    trans_status = Column(String(1, 'utf8mb4_unicode_ci'))
    trans_status_reason = Column(String(45, 'utf8mb4_unicode_ci'))
    eci = Column(String(1, 'utf8mb4_unicode_ci'))
    error_code = Column(String(45, 'utf8mb4_unicode_ci'))
    error_component = Column(String(45, 'utf8mb4_unicode_ci'))
    error_msg = Column(String(45, 'utf8mb4_unicode_ci'))
    create_time = Column(TIMESTAMP(fsp=6), nullable=False,
                         server_default=text("CURRENT_TIMESTAMP(6)"))

    def __init__(self, **kwargs):
        for property, value in kwargs.items():

            setattr(self, property, value)


class ThreeDs2RealResultBacklog(Base):
    __tablename__ = 'tb_3ds2_real_result_backlog'

    pk_id = Column(Integer, primary_key=True,
                   autoincrement=True, nullable=False)
    sn = Column(String(45, 'utf8mb4_unicode_ci'))
    return_code = Column(String(10, 'utf8mb4_unicode_ci'))
    create_time = Column(TIMESTAMP(fsp=6), nullable=False,
                         server_default=text("CURRENT_TIMESTAMP(6)"))


class TradeAuthorizationRealResult(Base):
    __tablename__ = 'tb_auth_real_result'

    pk_id = Column(Integer, primary_key=True,
                   autoincrement=True, nullable=False)
    return_code = Column(String(10, 'utf8mb4_unicode_ci'))
    client_info_id = Column(String(100, 'utf8mb4_unicode_ci'))
    sn = Column(String(60, 'utf8mb4_unicode_ci'))
    customer_servertime = Column(TIMESTAMP(fsp=6), nullable=False)
    auth_return_code = Column(String(10, 'utf8mb4_unicode_ci'))
    create_time = Column(TIMESTAMP(fsp=6), nullable=False,
                         server_default=text("CURRENT_TIMESTAMP(6)"))

    def __init__(self, **kwargs):
        for property, value in kwargs.items():

            setattr(self, property, value)


class TradeAuthorizationRealResultBacklog(Base):
    __tablename__ = 'tb_auth_real_result_backlog'

    pk_id = Column(Integer, primary_key=True,
                   autoincrement=True, nullable=False)
    sn = Column(String(45, 'utf8mb4_unicode_ci'))
    return_code = Column(String(10, 'utf8mb4_unicode_ci'))
    create_time = Column(TIMESTAMP(fsp=6), nullable=False,
                         server_default=text("CURRENT_TIMESTAMP(6)"))



from sqlalchemy import Column, Integer, TIMESTAMP, Float, text
from sqlalchemy.dialects.mysql import VARCHAR
from sqlalchemy.ext.declarative import declarative_base


Base = declarative_base()
# metadata = Base.metadata

print("----")
class TbmerchantriskValidationReport(Base):
    __tablename__ = 'tb_merchantrisk_validation_report'

    pk_id = Column(Integer, primary_key=True)
    customer_id = Column(VARCHAR(20))
    model_id = Column(VARCHAR(20))
    training_id = Column(VARCHAR(20))
    source_start_date = Column(VARCHAR(20))
    source_end_date = Column(VARCHAR(20))
    model_name = Column(VARCHAR(100))
    training_sample_count = Column(Integer)
    validation_sample_count = Column(Integer)
    accuracy = Column(Float)
    precision = Column(Float)
    recall = Column(Float)
    f1_score = Column(Float)
    merchantrisk_weight = Column(Float)
    merchantrisk_accuracy = Column(Float)
    merchantrisk_precision = Column(Float)
    merchantrisk_recall = Column(Float)
    merchantrisk_f1_score = Column(Float)
    create_time = Column(TIMESTAMP, server_default=text("CURRENT_TIMESTAMP(6)"))


class TBmerchantRiskInfo(Base):
    __tablename__ = 'tb_merchant_risk_info'

    pk_id = Column(Integer, primary_key=True)
    serial_number = Column(VARCHAR(100),default='undefine')
    diia_serial_number = Column(VARCHAR(100),default='undefine')
    institute_id = Column(VARCHAR(255),default='undefine')
    operator_id = Column(VARCHAR(255),default='undefine')
    connector_id = Column(VARCHAR(255),default='undefine')
    customer_id = Column(VARCHAR(20))
    profile_id = Column(VARCHAR(20))
    model_id = Column(VARCHAR(20))
    training_id = Column(VARCHAR(20))
    tag = Column(VARCHAR(20))
    data_version = Column(VARCHAR(10))
    mac = Column(VARCHAR(100))
    timestamp = Column(VARCHAR(100))
    merchant_id = Column(VARCHAR(100),default='undefine')
    pan_hash = Column(VARCHAR(100),default='undefine')
    purchase_amount = Column(VARCHAR(100),default='undefine')
    purchase_currency = Column(Integer, nullable=False,default=999)
    merchant_create_time = Column(VARCHAR(50),default='undefine')
    create_time = Column(TIMESTAMP, nullable=False, server_default=text("CURRENT_TIMESTAMP(6)"))


class TBmerchantRiskPredictresult(Base):
    __tablename__ = 'tb_merchant_risk_predict_result'

    pk_id = Column(Integer, primary_key=True)
    mr_info_id = Column(Integer, nullable=False)
    serial_number = Column(VARCHAR(100),default='undefine')
    # service_report_return_code = Column(VARCHAR(10))
    # service_report_score = Column(Float)
    # service_report_weight = Column(Float)
    # service_report_ahp_score = Column(Float)
    # service_report_reason = Column(VARCHAR(100))
    merchant_risk_return_code = Column(VARCHAR(10))
    merchant_risk_score = Column(Float)
    merchant_risk_weight = Column(Float)
    merchant_risk_ahp_score = Column(Float)
    merchant_risk_reason = Column(VARCHAR(100))
    total_score = Column(Float)
    policy_score = Column(Float)
    create_time = Column(TIMESTAMP, server_default=text("CURRENT_TIMESTAMP(6)"))




import time
import pandas as pd
from sqlalchemy.pool import QueuePool
from sqlalchemy import asc, desc, create_engine
from sqlalchemy.orm import sessionmaker, scoped_session, load_only


def get_orm_profile(host='192.168.10.201', db='acqfd_test', **kwargs):

    profile = {
        'host': host,
        'port': '3306',
        'user': 'acqfd',
        'pwd': 'acqfd16313302',
        'db': db
    }

    if kwargs:
        profile.update(kwargs)

    return profile


class Orm():
    def __init__(
        self,
        profile=get_orm_profile(),
        pool=False,
        pool_size=250,
        max_overflow=0,
        pool_recycle=10,
        pool_timeout=10,
    ):
        self.profile = profile
        self.sql_connect = f"mysql+pymysql://{profile['user']}:{profile['pwd']}@{profile['host']}:{profile['port']}/{profile['db']}?charset=utf8mb4&binary_prefix=true"
        if pool:
            engine = create_engine(
                self.sql_connect,
                pool_size=pool_size,
                max_overflow=max_overflow,
                pool_recycle=pool_recycle,
                pool_timeout=pool_timeout,
                pool_pre_ping=True,
                poolclass=QueuePool,
                pool_use_lifo=True
            )
        else:
            engine = create_engine(self.sql_connect, echo=False)
        self._sessionmaker = sessionmaker(bind=engine)
        self.session = scoped_session(self._sessionmaker)

    def get_session(self):
        return scoped_session(self._sessionmaker)

    def create_table(self, base, table):
        '''Create a table in DB'''

        engine = create_engine(self.sql_connect, echo=True)
        table
        base.metadata.create_all(engine)

    def delete(self, data: pd.DataFrame, tablename):
        '''Delete data in DB'''

        session = self.get_session()
        for i in range(data.shape[0]):
            kwargs = data.iloc[i].to_dict()
            query_data = session.query(tablename).filter_by(**kwargs).all()
            for row in query_data:
                session.delete(row)
                session.commit()
        session.close()
        
    def get_log_position(self):
        result = pd.read_sql(f"show master status;", self.session.bind)
        position = result['Position'].values[0]
        return position
    
    def check_sn(self, tablecolumn, sn: str):
        '''Check if a sn is in a table'''

        session = self.get_session()
        try:
            check_sn = session.query(tablecolumn).filter_by(sn=sn).all()
            session.close()

            if len(check_sn) == 0:
                return True
            return False

        except:
            session.rollback()

    def query_filter(self, tablename, limit: int = None, order_by: tuple = None, fields: list = None, *args):
        '''Get data from DB'''

        session = self.get_session()
        query = session.query(tablename).filter(*args)

        if order_by:
            if order_by[1] == 'asc':
                query = query.order_by(asc(order_by[0]))
            else:
                query = query.order_by(desc(order_by[0]))

        if limit:
            query = query.limit(limit)

        if fields:
            query = query.options(load_only(*fields))

        result = pd.read_sql(query.statement, session.bind)
        session.close()

        return result

    def elk_query_filter(self, tablename, limit=None, *args):
        query = self.session.query(tablename).filter(*args)
        if limit:
            query = query.limit(limit)
        return pd.read_sql(query.statement, self.session.bind)

    def update(self, tablename, column_values: dict = {}, update_content: dict = {}):
        '''用於更新cell的內容'''
        session = self.get_session()
        session.query(tablename).filter_by(
            **column_values).update(dict(**update_content))
        session.commit()
        session.close()

    def check_exist(self, tablename, **kwargs):
        '''用於確認指定資料是否存在'''
        session = self.get_session()
        q = session.query(tablename).filter_by(**kwargs)
        check_result = session.query(q.exists()).scalar()
        session.close()
        return check_result

    def __get_table_row(self, table, row: dict):
        '''Convert a row of data to an ORM object'''
        return table(**row)

    def data_to_DB(self, df: pd.DataFrame, table, start=0):
        '''Import data to DB'''

        # 轉換時間格式
        for col in df.columns:
            if df[col].dtype == pd._libs.tslibs.timestamps.Timestamp:
                df[col] = df[col].astype(str).apply(
                    lambda x: None if x == 'NaT' else x)

        # 補空值
        if df.isnull().sum().sum() != 0:
            df = df.where(pd.notnull(df), None)

        datarows = df.to_dict('records')

        session = self.get_session()

        t1 = time.time()
        alldata = []
        N = len(datarows)
        for i, row in enumerate(datarows):
            if i >= start:
                data = self.__get_table_row(table, row)
                alldata.append(data)

            n = i+1
            progress = f"\r|{'█'*int(n*50/N)}{' '*(50-int(n*50/N))} | {n}/{N} ({round(n/N*100, 2)}%)"
            print(progress, end='')

        print('\nUploading...')
        session.add_all(alldata)
        session.commit()
        session.close()
        t2 = time.time()
        print('Done.\n')

        total = round(t2-t1, 4)
        print(f'Total process time = {total} ({total/N} secondes/data)')


def main():
    from sqlalchemy import Column, text, Integer
    from sqlalchemy.dialects.mysql import TIMESTAMP
    from sqlalchemy.ext.declarative import declarative_base

    orm = Orm(
        profile={
            'host': '192.168.10.201',
            'port': '3306',
            'user': 'acqfd',
            'pwd': 'acqfd16313302',
            'db': 'acqfd_test'
        }
    )

    Base = declarative_base()
    metadata = Base.metadata

    class TestTable(Base):
        __tablename__ = 'udid_history'

        pk_id = Column(Integer, primary_key=True,
                       autoincrement=True, unique=True)
        create_time = Column(TIMESTAMP(fsp=6), nullable=False,
                             server_default=text("CURRENT_TIMESTAMP(6)"))

    # 建立資料表
    orm.create_table(Base, TestTable)
    print('Create table done.')


if __name__ == "__main__":
    main()



import os
import shutil
import xml.etree.ElementTree as ET
from tqdm import tqdm

type_arg = {
    "origin_file_lab": './training/image_3/',
    "target_file_lab": "./training/image_2/",
    "origin_file_img": "./training/label_3/",
    "target_file_img": "./training/label_2/",
    "data_size": 1000,
    "img_type": "jpg",
}


class OSOperate:
    
    def __init__(self, type_arg):
        self.origin_file_lab = type_arg.origin_file_lab
        self.target_file_lab = type_arg.target_file_lab
        self.origin_file_img = type_arg.origin_file_img
        self.target_file_img = type_arg.target_file_img
        self.data_size = type_arg.data_size
        self.img_type = type_arg.img_type
        self.ls_images = []

    def copy_img(self):
        ls_images = os.listdir(self.origin_file_img)
        if self.data_size < 0:
            ls_images = ls_images[self.data_size:]
        elif self.data_size > 0:
            ls_images = ls_images[:self.data_size]
        self.ls_images = ls_images
        
        shutil.rmtree(self.target_file_img)
        if not os.path.exists(self.target_file_img):
            os.mkdir(self.target_file_img)
        for img in ls_images:
            shutil.copyfile(self.origin_file_img + img, self.target_file_img + img)

    def copy_label(self):
        shutil.rmtree(self.target_file_lab)

        if not os.path.exists(self.target_file_lab):
            os.mkdir(self.target_file_lab)

        for img in self.ls_images:
            img = img.replace(self.img_type, "txt")
            shutil.copyfile(self.origin_file_lab + img, self.target_file_lab + img)

    def copy_file(self):
        self.copy_img()
        self.copy_label()

    def rename_allfile_type(self, path_file='./testing/image_2/', str_type="jpg"):
        ls_images = os.listdir(path_file)
        for img in ls_images:
            file_name = img.split(".")[0]
            os.rename(path_file + img, path_file + file_name + "." + self.img_type)


class XMLtoDeepstreamFormat:
    def __init__(self, base_xml_dir, kitti_saved_dir):
        self.base_xml_dir = base_xml_dir
        self.kitti_saved_dir = kitti_saved_dir
        
    def convert_annotation(self, file_name):
        in_file = open(self.base_xml_dir + file_name)
        tree = ET.parse(in_file)
        root = tree.getroot()

        with open(self.kitti_saved_dir + file_name[:-4] + '.txt', 'w') as f:
            for obj in root.iter('object'):
                cls = obj.find('name').text
                xmlbox = obj.find('bndbox')
                """
                    第5～8这4个数：物体的2维边界框
                    xmin，ymin，xmax，ymax
                """
                xmin, ymin, xmax, ymax = xmlbox.find('xmin').text, xmlbox.find('ymin').text, xmlbox.find('xmax').text, xmlbox.find('ymax').text
                # if cls == 'trafficsignal' or 'trafficlight':
                #     cls = 'road_sign'
                # elif cls == 'car' or 'bus':
                #     cls = 'vehicle'
                # elif cls == 'bicycle' or 'motorbike':
                #     cls = 'bicycle'
                # elif cls == 'person':
                #     cls = 'pedestrain'
                # else:
                #     pass

                f.write(cls + " " + '0.00' + " " + '0' + " " + '0.0' + " " + str(xmin) + " " + str(ymin) + " " + str(xmax) + " " + str(ymax) + " " +
                        '0.0' + " " + '0.0' + " " + '0.0' + " " + '0.0' + " " + '0.0' + " " + '0.0' + " " + '0.0' + '\n')
    
    def run(self):
        xml_list = os.listdir(self.base_xml_dir)
        for i in tqdm(range(len(xml_list))):
            self.convert_annotation(xml_list[i])


if __name__ == "__main__":
    # 格式轉換
    base_xml_dir = "/workspace/tlt-experiments/data/training/label_3/"
    kitti_saved_dir = "/workspace/tlt-experiments/data/training/label_2/"
    xmltodp = XMLtoDeepstreamFormat(base_xml_dir, kitti_saved_dir)
    xmltodp.run()
    # ====================================================================
    osop = OSOperate(type_arg)
    osop.copy_file()

    osop.rename_allfile_type(path_file='./testing/image_2/', str_type="jpg")




#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time:2020.09.12
# @author:xhrg
# @email:634789257@qq.com

import json
import os
import threading
import inspect
import ctypes
import time

from .util import *

version = sys.version_info.major

if version == 2:
    from .python_2x import *

if version == 3:
    from .python_3x import *


class ApolloClient(object):

    def __init__(self, config_url, app_id, cluster='default', secret='', start_hot_update=True, start_heartbeat=True,
                 change_listener=None):

        # 核心路由参数
        self.config_url = config_url
        self.cluster = cluster
        self.app_id = app_id

        # 非核心参数
        self.ip = init_ip()
        self.secret = secret

        # 检查参数变量

        # 私有控制变量
        self._cycle_time = 2
        self._stopping = False
        self._cache = {}
        self._no_key = {}
        self._hash = {}
        self._pull_timeout = 75
        self._cache_file_path = os.path.expanduser('~') + '/data/apollo/cache/'
        self._long_poll_thread = None
        self._change_listener = change_listener  # "add" "delete" "update"
        self._notification_map = {'application': -1}
        self.last_release_key = None
        # 私有启动方法
        self._path_checker()
        if start_hot_update:
            self._start_hot_update()

        # 启动心跳线程
        if start_heartbeat:
            heartbeat = threading.Thread(target=self._heartBeat)
            heartbeat.setDaemon(True)
            heartbeat.start()

    def get_json_from_net(self, namespace='application'):
        url = '{}/configs/{}/{}/{}?releaseKey={}&ip={}'.format(self.config_url, self.app_id, self.cluster, namespace,
                                                               self.secret, self.ip)
        try:
            code, body = http_request(url, timeout=3, headers=self._signHeaders(url))
            if code == 200:
                data = json.loads(body)
                data = data["configurations"]
                return_data = {CONFIGURATIONS: data}
                return return_data
            else:
                return None
        except Exception as e:

            return None

    def get_value(self, key, default_val=None, namespace='application'):
        try:
            # 读取内存配置
            namespace_cache = self._cache.get(namespace)
            val = get_value_from_dict(namespace_cache, key)
            if val is not None:
                return val

            no_key = no_key_cache_key(namespace, key)
            if no_key in self._no_key:
                return default_val

            # 读取网络配置
            namespace_data = self.get_json_from_net(namespace)
            val = get_value_from_dict(namespace_data, key)
            if val is not None:
                self._update_cache_and_file(namespace_data, namespace)
                return val

            # 读取文件配置
            namespace_cache = self._get_local_cache(namespace)
            val = get_value_from_dict(namespace_cache, key)
            if val is not None:
                self._update_cache_and_file(namespace_cache, namespace)
                return val

            # 如果全部没有获取，则把默认值返回，设置本地缓存为None
            self._set_local_cache_none(namespace, key)
            return default_val
        except Exception as e:
            return default_val

    # 设置某个namespace的key为none，这里不设置default_val，是为了保证函数调用实时的正确性。
    # 假设用户2次default_val不一样，然而这里却用default_val填充，则可能会有问题。
    def _set_local_cache_none(self, namespace, key):
        no_key = no_key_cache_key(namespace, key)
        self._no_key[no_key] = key

    def _start_hot_update(self):
        self._long_poll_thread = threading.Thread(target=self._listener)
        # 启动异步线程为守护线程，主线程推出的时候，守护线程会自动退出。
        self._long_poll_thread.setDaemon(True)
        self._long_poll_thread.start()

    def stop(self):
        self._stopping = True

    # 调用设置的回调函数，如果异常，直接try掉
    def _call_listener(self, namespace, old_kv, new_kv):
        if self._change_listener is None:
            return
        if old_kv is None:
            old_kv = {}
        if new_kv is None:
            new_kv = {}
        try:
            for key in old_kv:
                new_value = new_kv.get(key)
                old_value = old_kv.get(key)
                if new_value is None:
                    # 如果newValue 是空，则表示key，value被删除了。
                    self._change_listener("delete", namespace, key, old_value)
                    continue
                if new_value != old_value:
                    self._change_listener("update", namespace, key, new_value)
                    continue
            for key in new_kv:
                new_value = new_kv.get(key)
                old_value = old_kv.get(key)
                if old_value is None:
                    self._change_listener("add", namespace, key, new_value)
        except BaseException as e:
            pass

    def _path_checker(self):
        if not os.path.isdir(self._cache_file_path):
            makedirs_wrapper(self._cache_file_path)


    # 更新本地缓存和文件缓存
    def _update_cache_and_file(self, namespace_data, namespace='application'):
        # 更新本地缓存
        self._cache[namespace] = namespace_data
        # 更新文件缓存
        new_string = json.dumps(namespace_data)
        new_hash = hashlib.md5(new_string.encode('utf-8')).hexdigest()
        if self._hash.get(namespace) == new_hash:
            pass
        else:
            with open(os.path.join(self._cache_file_path, '%s_configuration_%s.txt' % (self.app_id, namespace)),
                      'w') as f:
                f.write(new_string)
            self._hash[namespace] = new_hash

    # 从本地文件获取配置
    def _get_local_cache(self, namespace='application'):
        cache_file_path = os.path.join(self._cache_file_path, '%s_configuration_%s.txt' % (self.app_id, namespace))
        if os.path.isfile(cache_file_path):
            with open(cache_file_path, 'r') as f:
                result = json.loads(f.readline())
            return result
        return {}

    def _long_poll(self):
        notifications = []
        for key in self._cache:
            namespace_data = self._cache[key]
            notification_id = -1
            if NOTIFICATION_ID in namespace_data:
                notification_id = self._cache[key][NOTIFICATION_ID]
            notifications.append({
                NAMESPACE_NAME: key,
                NOTIFICATION_ID: notification_id
            })
        try:
            # 如果长度为0直接返回
            if len(notifications) == 0:
                return
            url = '{}/notifications/v2'.format(self.config_url)
            params = {
                'appId': self.app_id,
                'cluster': self.cluster,
                'notifications': json.dumps(notifications, ensure_ascii=False)
            }
            param_str = url_encode_wrapper(params)
            url = url + '?' + param_str
            code, body = http_request(url, self._pull_timeout, headers=self._signHeaders(url))
            http_code = code
            if http_code == 304:
                return
            if http_code == 200:
                data = json.loads(body)
                for entry in data:
                    namespace = entry[NAMESPACE_NAME]
                    n_id = entry[NOTIFICATION_ID]
                    self._get_net_and_set_local(namespace, n_id, call_change=True)
                    return
            else:
                pass
        except Exception as e:
            pass

    def _get_net_and_set_local(self, namespace, n_id, call_change=False):
        namespace_data = self.get_json_from_net(namespace)
        namespace_data[NOTIFICATION_ID] = n_id
        old_namespace = self._cache.get(namespace)
        self._update_cache_and_file(namespace_data, namespace)
        if self._change_listener is not None and call_change:
            old_kv = old_namespace.get(CONFIGURATIONS)
            new_kv = namespace_data.get(CONFIGURATIONS)
            self._call_listener(namespace, old_kv, new_kv)

    def _listener(self):
        while not self._stopping:
            self._long_poll()
            time.sleep(self._cycle_time)

    # 给header增加加签需求
    def _signHeaders(self, url):
        headers = {}
        if self.secret == '':
            return headers
        uri = url[len(self.config_url):len(url)]
        time_unix_now = str(int(round(time.time() * 1000)))
        headers['Authorization'] = 'Apollo ' + self.app_id + ':' + signature(time_unix_now, uri, self.secret)
        headers['Timestamp'] = time_unix_now
        return headers

    def _heartBeat(self):
        while not self._stopping:
            for namespace in self._notification_map:
                self._do_heartBeat(namespace)
            time.sleep(60 * 10)  # 10分钟

    def _do_heartBeat(self, namespace):
        url = '{}/configs/{}/{}/{}?releaseKey={}&ip={}'.format(self.config_url, self.app_id, self.cluster, namespace,
                                                               self.secret, self.ip)
        try:
            code, body = http_request(url, timeout=3, headers=self._signHeaders(url))
            if code == 200:
                data = json.loads(body)
                if self.last_release_key == data["releaseKey"]:
                    return None
                self.last_release_key = data["releaseKey"]
                data = data["configurations"]
                self._update_cache_and_file(data, namespace)
            else:
                return None
        except Exception as e:
            return None



import json
from kafka import KafkaProducer


class MessageQueue:
    def __init__(self, bootstrap_servers, customer_id, model_id, training_id, topic, KAFKA_USE, init_logger):
        self.bootstrap_servers = bootstrap_servers
        self.customer_id = customer_id
        self.model_id = model_id
        self.training_id = training_id
        self.topic = topic
        self.KAFKA_USE = KAFKA_USE
        self.startup_key = "MQ0003"
        self.status_key = "MQ0004"
        self.init_logger = init_logger

    def _produce(self, key, value):
        self.init_logger.info("MQ Send: " + str(value))

        # send MQ for service start successfully
        if self.KAFKA_USE == "true":
            self.init_logger.info("KAFKA_USE=true")
            producer = KafkaProducer(bootstrap_servers=self.bootstrap_servers)

            data = json.dumps(value).encode('utf-8')
            producer.send(self.topic, key=key.encode('utf-8'), value=data)
            producer.flush()
            producer.close()
        else:
            self.init_logger.info("KAFKA_USE=false")

    def startup(self, return_code, reason):
        value = {
            "return_code": return_code,
            "payload": {
                "customer_id": self.customer_id,
                "model_id": self.model_id,
                "training_id": self.training_id,
                "progress": "0",
                "reason": reason,
            }
        }
        self._produce(self.startup_key, value)

    def send(self, return_code, progress, reason):
        value = {
            "return_code": return_code,
            "payload": {
                "customer_id": self.customer_id,
                "model_id": self.model_id,
                "training_id": self.training_id,
                "progress": progress,
                "reason": reason,
            }
        }
        self._produce(self.status_key, value)

    def finish(self, return_code, progress, kg_list, val_report, reason):
        value = {
            "return_code": return_code,
            "payload": {
                "customer_id": self.customer_id,
                "model_id": self.model_id,
                "training_id": self.training_id,
                "progress": progress,
                "kg_list": kg_list,
                "val_report": val_report,
                "reason": reason,
            }
        }
        self._produce(self.status_key, value)



from setuptools import setup
import os
import subprocess

p = subprocess.Popen(args='pip install -r requirements.txt', shell=True)


def get_all_dir(rootdir):
    list_file = []
    for file in os.listdir(rootdir):
        d = os.path.join(rootdir, file)
        if os.path.isdir(d) and "__pycache__" not in d:
            print(d)
            try:
                list_file.append(d.split("./")[1])
            except Exception:
                list_file.append(d.split("./")[0])
    return list_file


list_file = []
rootdir = './hitrustai_lab'
for i in get_all_dir(rootdir):
    list_file.append(i)
    list_r = get_all_dir(i)
    if len(list_r) > 0:
        for ii in list_r:
            list_file.append(ii)

setup(
    name='hitrustai_lab',
    version='1.0.32',

    url='https://gitlab.hitrustai.com/bruce60108010204/hitrustai-lab',
    author='Bruce Tsai',
    author_email='brcue60108010204@gmail.com',
    packages=list_file,
)



import maxminddb


class Geoip:
    def __init__(self, path):
        self.reader1 = maxminddb.open_database(path)

    def get_diia(self, ip):
        re = self.reader1.get(ip)['diia']
        return re

    def get_mmdb(self, ip):
        re = self.reader1.get(ip)
        return re
